// LambdaAusdruecke.cpp: Hauptprojektdatei.
#include <iostream>
#include <vector>
#include <algorithm>
#include <cstdlib>
using namespace std;

int main()
{

   // Beispiel fr einen mglichst einfachen Lambda-Ausdruck
   auto gruss = [] { cout << "Hallo" << endl; };
   gruss();                                                    // Ausgabe: Hallo

   cout << endl;


   // Lambda-Ausdruck, der einen Wert zurckliefert
   int n = [] { return 3; }();
   cout << n << endl;                                          // Ausgabe: 3

   cout << endl;


   // Lambda-Ausdruck, der Argumente bernimmt
   int m1 = [] (int a, int b) { return a + b; }(5, 1);
   int m2 = [] (int a, int b) { return a + b; }(5, 2);
   cout << m1 << " " << m2 << endl;                            // Ausgabe: 6 7

   cout << endl;


   // Lambda-Ausdruck, der auf lokale Variable zugreift
   int faktor = 2;
   int m3 = [=] (int a) { return faktor * a; }(5);
   cout << m3 << endl;                                         // Ausgabe: 10

   cout << endl;


   // Lambda-Ausdruck, der Rckgabetyp spezifiziert
   faktor = 2;
   auto test = [=] (int a) -> bool 
   { 
      if (faktor * a > 10)
         return true;
      else
         return false;
   };
   cout << boolalpha << test(3) << " " << test(6) << endl;     // Ausgabe: false true

   cout << endl;


   // Lambda-Ausdruck, der kopierte Variablen verndert
   int zahl = 2;
   auto demo = [=] () mutable 
                   { 
                     zahl = 10;
                     cout << zahl << " ";                      // Ausgabe: 10
                   };
   demo();
   cout << zahl << endl;                                       // Ausgabe: 2
   



   {
   // Lambda-Ausdruck als Funktionsobjekt-Ersatz
   vector<double> werte;
   typedef vector<double>::iterator iter;

   for (int i = 0; i < 5; i++)
      werte.push_back(i); 
   
   int n = 5;
   generate(werte.begin(), werte.end(), [=] { return rand() * n; }  );
   
   for (iter it = werte.begin(); it != werte.end(); it++)
      cout << *it << " ";                     
   }

   cout << endl;
   return 0;
}
